package com.loafle.overflow.central.module.email.service;


import com.loafle.overflow.central.commons.utils.EmailSender;
import com.loafle.overflow.central.commons.utils.GenerateKey;
import com.loafle.overflow.central.module.domain.dao.DomainDAO;
import com.loafle.overflow.central.module.domain.dao.DomainMemberDAO;
import com.loafle.overflow.central.module.email.dao.EmailAuthDAO;

import com.loafle.overflow.central.module.member.dao.MemberDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.Mail;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.domain.DomainMember;
import com.loafle.overflow.model.email.EmailAuth;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.meta.MetaEmailStatus;
import com.loafle.overflow.model.meta.MetaMemberStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailException;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.*;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("EmailAuthService")
public class EmailAuthService {
    private static final String WEB_GO_MEMBER_ADDR = "http://127.0.0.1:19080/account/";

    @Autowired
    private EmailAuthDAO emailAuthDAO;

    @Autowired
    private MemberDAO memberDAO;

    @Autowired
    private DomainDAO domainDAO;

    @Autowired
    private DomainMemberDAO domainMemberDAO;

    @Autowired
    private EmailSender emailSender;


//    private static final String WEB_NG_MEMBER_ADDR = "http://127.0.0.1:4200/auth/";

    public EmailAuth read(long id) {
        return this.emailAuthDAO.findOne(id);
    }

    public EmailAuth readBySignupAuthKey(String token) throws OverflowException {
//        System.out.println("authKey = [" + token + "]");
//        String deStr = URLDecoder.decode(token, "UTF-8");
//        System.out.println("deStr = [" + deStr + "]");
//
//        EmailAuth auth = this.emailAuthDAO.findByEmailAuthKey(deStr);

        EmailAuth auth = this.readByToken(token);

        if (auth != null) {

            boolean res = this.isValidateTime(auth);

            if (!res) {
                throw new OverflowException("The authentication expiration time has passed.", new Throwable());
            }

            auth.setAuthConfirmDate(new Date());
            this.emailAuthDAO.save(auth);
            auth.getMember().setStatus(new MetaMemberStatus((short)2));
            this.memberDAO.save(auth.getMember());

            Domain domain = new Domain();
            domain.setName(auth.getMember().getCompanyName());
            Domain domain1 = this.domainDAO.save(domain);

            DomainMember domainMember = new DomainMember();
            domainMember.setDomain(domain1);
            domainMember.setMember(auth.getMember());
            this.domainMemberDAO.save(domainMember);
        }

        return auth;
    }

    public EmailAuth readByPwAuthKey(String token) throws OverflowException {

        EmailAuth auth = this.readByToken(token);

        if (auth != null && (auth.getMember() != null && auth.getMember().getId() > 0)) {
            boolean res = this.isValidateTime(auth);

            if (!res) {
                throw new OverflowException("The authentication expiration time has passed.", new Throwable());
            }

            auth.setAuthConfirmDate(new Date());
            this.emailAuthDAO.save(auth);
        }

        return auth;

    }

    public EmailAuth readByToken(String token) {
        String encode = "";

        try {
            encode = URLEncoder.encode(token, "UTF-8");
        }catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        EmailAuth auth = this.emailAuthDAO.findByEmailAuthKey(encode);

        return auth;
    }
    // dZQgXM1o/Cx48X8DM+6ec/oPfqA2l/LdWtijOZ2EnWk=

    public List<EmailAuth> readByMember(long memberId) {
        return this.emailAuthDAO.findByMember(new Member(memberId));
    }

    public EmailAuth modify(EmailAuth emailAuth) {
        return this.emailAuthDAO.save(emailAuth);
    }

    public EmailAuth sendEmailByMember(Member member) throws OverflowException {
        return this.sendEMail(member, 1);
    }

    // Todo Send Email Refactoring
    public EmailAuth sendEmailResetPassword(Member member) throws OverflowException {
        return this.sendEMail(member, 2);
    }

    private boolean isValidateTime(EmailAuth auth) {

        Calendar cal = Calendar.getInstance();

        cal.setTime(auth.getCreateDate());
        cal.add(Calendar.HOUR, 12);
        Date futureDate = cal.getTime();

        Date nowDate = new Date();

        if (!nowDate.before(futureDate)) {
            return false;
        }

        return true;
    }

    private EmailAuth sendEMail(Member member, int status) throws OverflowException {
        String key = GenerateKey.getKey();
//        String enMail = emailSender.encrypt(key);

        String mailSubject = null;
        String entry = null;
        String templateName = null;

        EmailAuth auth = new EmailAuth();
        auth.setMember(member);
        auth.setEmailAuthKey(key);
        auth.setEmailStatus(new MetaEmailStatus(status));

//        System.out.println("encode = [" + encode + "]");
        String encode = "";

        try {
            encode = URLEncoder.encode(key, "UTF-8");
        }catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        if (status == 1) {
            mailSubject = "Signup Confirm Mail";
            entry = "confirm_email";
            templateName = "/vmtemplates/signup.vm";
        } else if (status == 2) {
            mailSubject = "Reset Password Confirm Mail";
            entry = "confirm_reset_pw";
            templateName = "/vmtemplates/password_reset.vm";
        }

        Mail mail = new Mail();
        mail.setMailTo(member.getEmail());
        mail.setMailSubject(mailSubject);
        mail.setTemplateLoacation(templateName);

        String uri = WEB_GO_MEMBER_ADDR + entry + "?token=" + encode;

        Map<String, Object> model = new HashMap<>();
        model.put("firstName", auth.getMember().getName());
        model.put("lastName", auth.getMember().getCompanyName());
        model.put("location", "Seoul");
        model.put("signature", "www.loafle.com");
        model.put("content", uri);
        mail.setModel(model);
        emailSender.sendSimpleEmail(mail);

        this.emailAuthDAO.save(auth);

        return auth;
    }
}



//    EmailAuth auth = new EmailAuth();
//        auth.setMember(member);
//                String en = emailSender.encrypt(member.getEmail());
//                auth.setEmailAuthKey(en);
//                auth.setEmailStatus(new MetaEmailStatus(1));
//                String encode = URLEncoder.encode(en, "UTF-8");
//
////        System.out.println("encode = [" + encode + "]");
//                Mail mail = new Mail();
//                mail.setMailTo(member.getEmail());
//                mail.setMailSubject("Confirm Email");
////        mail.setMailContent("http://127.0.0.1:19080/account/confirm_signup?key="+ encode +"\r\nConfirm Email");
//
//                String uri = WEB_GO_MEMBER_ADDR + "confirm_signup?key=" + encode + "\r\nConfirm Email";
//                Map<String, Object> model = new HashMap<>();
//        model.put("firstName", auth.getMember().getName());
//        model.put("lastName", auth.getMember().getCompanyName());
//        model.put("location", "Seoul");
//        model.put("signature", "www.loafle.com");
//        model.put("content", uri);
//
//        mail.setModel(model);
//        emailSender.sendSimpleEmail(mail);
//
//        this.emailAuthDAO.save(auth);
//
//        return auth;