package com.loafle.overflow.central.module.probe.service;

import com.loafle.overflow.central.module.probe.dao.ProbeDAO;
import com.loafle.overflow.core.annotation.WebappAPI;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.service.central.probe.ProbeService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("ProbeService")
public class CentralProbeService implements ProbeService {

    @Autowired
    private ProbeDAO probeDAO;

    public Probe regist(Probe probe) throws OverflowException {
        return this.probeDAO.save(probe);
    }

    public List<Probe> regist(List<Probe> probes) throws OverflowException {
        return this.probeDAO.save(probes);
    }

    public List<Probe> readAllByDomain(Domain domain) throws OverflowException {
        List<Probe> probes = this.probeDAO.findAllByDomainOrderByIdDesc(domain);
        return probes;
    }

    public Probe read(long id) throws OverflowException {
        return this.probeDAO.findOne(id);
    }

    public Probe readByProbeKey(String probeKey) throws OverflowException {
        return this.probeDAO.findByProbeKey(probeKey);
    }

    public Probe modify(Probe probe) throws OverflowException {
        return this.probeDAO.save(probe);
    }

    public boolean remove(long id) throws OverflowException {
        this.probeDAO.delete(id);
        return true;
    }

    public Probe increaseTargetCount(Probe probe) {
        Probe p = this.probeDAO.findOne(probe.getId());
        p.setTargetCount(p.getTargetCount() + 1);
        return this.probeDAO.save(p);
    }

    public Probe decreaseTargetCount(Probe probe) {
        Probe p = this.probeDAO.findOne(probe.getId());
        int count = p.getTargetCount();
        if (count > 0) {
            p.setTargetCount(count - 1);
        }
        return this.probeDAO.save(p);
    }

    public Probe modifyDisplayName(long probeId, String displayName) {
        Probe probe = this.probeDAO.findOne(probeId);
        probe.setDisplayName(displayName);
        return this.probeDAO.save(probe);
    }

}
